// © 2023 John Breaux
//! A [Comment] stores the contents of a line comment, including the preceding `;` or `//`
use super::*;
#[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
pub struct Comment(pub String);

impl Parsable for Comment {
    fn parse<'text, T>(_: &Parser, stream: &mut T) -> Result<Self, Error>
    where T: TokenStream<'text> {
        let token = stream.expect(Type::Comment)?;
        Ok(Self(token.lexeme().to_string()))
    }
}
impl Display for Comment {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result { Display::fmt(&self.0, f) }
}
