// © 2023 John Breaux
//! An [Identifier] stores the hash of a named identifier
use super::*;
#[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
pub enum Identifier {
    Hash(Hash),
    Str(String),
}

impl Identifier {
    fn str<T: AsRef<str>>(s: T) -> Self { Self::Str(s.as_ref().into()) }
}

impl From<Hash> for Identifier {
    fn from(value: Hash) -> Self { Self::Hash(value) }
}

impl Parsable for Identifier {
    fn parse<'text, T>(_: &Parser, stream: &mut T) -> Result<Self, Error>
    where T: TokenStream<'text> {
        let token = stream.expect(Type::Identifier)?;
        match token.variant() {
            Type::Identifier => Ok(Self::str(token.lexeme())),
            _ => unreachable!("Expected identifier, got {token:?}"),
        }
    }
}
impl Display for Identifier {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            Identifier::Hash(_) => Display::fmt("Unresolved", f),
            Identifier::Str(s) => Display::fmt(s, f),
        }
    }
}
