//! Stores contextual information about the current tokenizer state, useful for printing errors
use super::*;
/// Stores contextual information about the current tokenizer state, useful for printing errors
#[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
pub struct Context {
    line: usize,
    tokens: usize,
    position: usize,
}

impl Context {
    pub fn new() -> Self { Default::default() }
    pub fn line(&self) -> usize { self.line }
    pub fn tokens(&self) -> usize { self.tokens }
    pub fn position(&self) -> usize { self.position }
    pub(super) fn count(&mut self, t: &Token) {
        match t.variant() {
            Type::EndOfFile => return,
            Type::Endl => {
                self.line += 1;
                self.position = 0;
            }
            _ => self.position += t.len(),
        }
        self.tokens += 1;
    }
}
impl Default for Context {
    fn default() -> Self { Self { line: 1, tokens: 0, position: 0 } }
}

impl std::fmt::Display for Context {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}:{}", self.line, self.position)
    }
}
